/*
Unreal Graphic Shader 1.0

Dev : Dimas Adiyaksa

Facebook : Dimas Adiyaksa
Fanspage Facebook : Unreal Graphic Shader PE
Instagram : @dimas.adiyaksa
Twitter : @dimasadiyaksa2
Discord : Dimas 956
*/

/*
CZM Saturation from :

https://github.com/CesiumGS/cesium/blob/master/Source/Shaders/Builtin/Functions/saturation.glsl

*/

lowp vec3 sat(vec3 rgb, float adjustment)
{
   const vec3 W = vec3(0.2125, 0.7154, 0.0721);
   vec3 intensity = vec3(dot(rgb, W));
return mix(intensity, rgb, adjustment);
}

/*
All Tonemap (Except Linear Tonemapping) from :

https://github.com/armory3d/armory/blob/master/Shaders/std/tonemap.glsl

*/

lowp vec3 uncharted2Tonemap(lowp vec3 x) 
{	
   float A = 1.0;
   float B = 0.6;
   float C = 0.4;
   float D = 0.5;
   float E = 0.02;
   float F = 0.9;	
return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

lowp vec3 tonemapUncharted2(lowp vec3 color) 
{	
   float W = 11.2;	
   float exposureBias = 2.0;	
   vec3 curr = uncharted2Tonemap(exposureBias * color);	
   vec3 whiteScale = 1.0 / uncharted2Tonemap(vec3(W));	
return curr * whiteScale;
}

lowp vec3 tonemapFilmic(lowp vec3 color) 
{	
   vec3 x = max(vec3(0.0), color - 0.004);	
return (x * (6.2 * x + 0.5)) / (x * (6.2 * x + 1.7) + 0.06);
}

lowp vec3 acesFilm(lowp vec3 x)
{ 
   float a = 1.0;
   float b = 0.1;
   float c = 1.0;
   float d = 0.05;
   float e = 0.7;
return clamp((x * (a * x + b)) / (x * (c * x + d ) + e), 0.0, 1.0);
}

/*
Linear Tonemapping from :

https://github.com/worleydl/hdr-shaders/blob/master/dolphin/hdr.glsl
*/

lowp vec3 linearToneMapping(lowp vec3 color)
{	
float gamma = 2.2; 
float exposure = 1.;
color = clamp(exposure * color, 0., 1.);	
color = pow(color, vec3(1. / gamma));
return color;
}

lowp vec3 tonemapping(lowp vec3 x)
{
   x = uncharted2Tonemap(x);
   x = tonemapUncharted2(x);
   x = acesFilm(x);
   x = tonemapFilmic(x);
return x;
}