// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

/*
Unreal Graphic Shader 1.0

Dev : Dimas Adiyaksa

Facebook : Dimas Adiyaksa
Fanspage Facebook : Unreal Graphic Shader PE
Instagram : @dimas.adiyaksa
Twitter : @dimasadiyaksa2
Discord : Dimas 956
*/

/*
=========== DONT STEAL ANYTHING !!! ==========
*/

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;
varying highp vec3 position;
varying highp vec3 wpos;

#ifdef FOG
varying vec4 fogColor;
#endif

#include "uniformShaderConstants.h"
#include "uniformPerFrameConstants.h"
#include "util.h"

#include "lib/Coloring.glsl"
#include "lib/Water.glsl"
#include "lib/Caustic.glsl"
#include "lib/Specular.glsl"

#include "Settings/Setting.ugspe"

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
#endif

//Get Normal (Code to Normal)
vec3 normal_texture = texture2D(TEXTURE_0, uv0).rgb;
normal_texture = normalize(normal_texture * 2. - 1.);
vec3 normal = normal_texture * .5 - .5;

vec4 fc = FOG_COLOR;
vec2 fcr = FOG_CONTROL;

//Detection
float tex = texture2D(TEXTURE_1, vec2(0, 1)).r;
tex = clamp((tex-0.5) * 2.0, 0.0, 1.0);
float day = tex;
float sun = .5-abs(.5-tex);
float night = 1.-tex;
float rain = (1. - pow(fcr.y, 11.)) * uv1.y;
float rain2 = (1. - pow(fcr.y, 11.));
float cave = 1. - uv1.y;
bool Nether = ( fc.r > fc.b && fc.r < 0.5 && fc.b < 0.2);
bool Unwater = (fcr.x == 0.);
bool Nwater = (color.b * 2. > color.r + color.g);
bool End = (fc.r > fc.g && fc.b > fc.g && fc.r < 0.05 && fc.b < 0.05 && fc.g < 0.05);
	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
	
vec4 inColor = color;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

//Night Coloring
#if !defined(ALWAYS_LIT)
   vec4 diff2 = texture2D(TEXTURE_1, uv1);
   vec3 night_color = vec3(1., 1., 1.5);
   diff2.rgb *= pow(diff2.rgb, 1. - night_color);
if(Nether == false){
   diffuse *= mix(diff2, texture2D(TEXTURE_1, uv1), 1.-uv1.y);
}
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif
	vec3 ao = inColor.rgb;
if(Unwater == false && End == false && Nether == false){
  ao = mix(ao, normalize(ao), cave);
}
	diffuse.rgb *= ao;
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

//Tonemapping
#ifdef Tonemap
diffuse.rgb = tonemapping(diffuse.rgb);
#endif

//Saturation
saturation = mix(saturation, rain_saturation, rain);
diffuse.rgb = sat(diffuse.rgb, saturation);

//Grayscale
float grayscale = dot(diffuse.rgb, vec3(0.299, 0.587, 0.114));
vec3 grayscale2 = vec3(.21 * diffuse.r + .71 * diffuse.g + .07 * diffuse.b);

//Grayscaling
vec3 gs = diffuse.rgb * 1.7 * grayscale;
diffuse.rgb = gs;

//Night Desaturation
vec3 blue_desaturation =  mix(vec3(.2, .4, 1.2) * grayscale, diffuse.rgb, max(0., uv1.x));
diffuse.rgb = mix(diffuse.rgb, blue_desaturation, night * uv1.y * (1. - rain));

//Rain Color
vec3 rain_color = diffuse.rgb * .5 * grayscale2;
diffuse.rgb = mix(diffuse.rgb, rain_color, rain * (1. - night) * (1. - uv1.x));

//Torch
if(Unwater == false && End == false){
vec3 torch_color_day = diffuse.rgb * vec3(1., .5, 0.);
vec3 torch_color_night = diffuse.rgb * vec3(1., .15, -1.) * 1.5;
vec3 torch_color_cave = diffuse.rgb * vec3(1.5, .35, -1.)*3.;
vec3 torch_color = mix(vec3(0.), torch_color_day, day * uv1.y);
torch_color = mix(torch_color, torch_color_night, night * uv1.y);
torch_color = mix(torch_color, torch_color_cave, cave);
float torch_radius = mix(1., max(0., uv1.x - .7), day * uv1.y);
torch_radius = mix(torch_radius, max(0., uv1.x - .5), night * uv1.y);
torch_radius = mix(torch_radius, max(0., uv1.x - .5), cave);
diffuse.rgb += torch_color * torch_radius;
}

//Under Water Lighting
if(Unwater == true)
{
vec3 l_color = mix(vec3(0.), vec3(.1, .3, .8), day);
l_color = mix(l_color, vec3(.0, .25, .5), night);
diffuse.rgb += l_color * max(0., uv1.x + .3);
}

//Shadow
vec3 shadow = mix(vec3(.93, .96, .95), vec3(1.), max(0.0, uv1.x));
if(uv1.y < .875)diffuse.rgb *= shadow;
if(uv1.y < .873)diffuse.rgb *= shadow;
if(uv1.y < .871)diffuse.rgb *= shadow;
if(uv1.y < .869)diffuse.rgb *= shadow;
if(uv1.y < .867)diffuse.rgb *= shadow;
if(uv1.y < .865)diffuse.rgb *= shadow;
if(uv1.y < .863)diffuse.rgb *= shadow;
if(uv1.y < .861)diffuse.rgb *= shadow;
if(uv1.y < .859)diffuse.rgb *= shadow;
if(uv1.y < .857)diffuse.rgb *= shadow;

//Direct Light (Side)
#ifdef DIRECTION_LIGHT
vec3 N = normalize(cross(dFdx(position), dFdy(position)));

vec3 shadow_s = vec3(.6, .8, .6);

#if defined(ALPHA_TEST) && defined(FANCY)
vec3 grass_side_day = mix(vec3(1.), shadow_s, max(0., N.x));
vec3 grass_side_sun = mix(vec3(1.), mix(vec3(1., .4, .0), vec3(1.), uv1.x), max(0., N.x));
vec3 grass_side_night = vec3(1.);

vec3 grass_side = mix(vec3(1.), grass_side_day, day * uv1.y);
grass_side = mix(grass_side, grass_side_sun, sun * uv1.y);
grass_side = mix(grass_side, grass_side_night, night);
diffuse.rgb *= mix(grass_side, vec3(1.), rain);

#else

vec3 side_block_day = mix(vec3(1.), shadow_s, max(0., max(N.x, -N.x)));
vec3 side_block_sun = mix(vec3(1.), mix(vec3(1., .5, 0.), vec3(1.), uv1.x), max(0., max(N.x, -N.x)));
side_block_sun = mix(side_block_sun, vec3(3.), max(0., max(N.z, -N.z)));
vec3 side_block_night = mix(vec3(1.), vec3(1.), max(0., max(N.z, -N.z)));
vec3 side_block_cave = mix(vec3(1.), vec3(.7), max(0., max(N.z, -N.z)));
side_block_cave = mix(side_block_cave, mix(vec3(.4), vec3(.8), uv1.x), max(0., max(N.x, -N.x)));

vec3 side_block = mix(vec3(1.), side_block_day, day * uv1.y);
side_block = mix(side_block, side_block_sun, sun * uv1.y);
side_block = mix(side_block, side_block_night, night * uv1.y);
side_block = mix(side_block, side_block_cave, cave);
if(Unwater == false)diffuse.rgb *= mix(side_block, vec3(1.), rain);
#endif
#endif

//Specular to Block
#if defined BLEND && defined SPECULAR_BLOCK
if(Nwater == false)diffuse += mix(vec4(0.), Specular(wpos), max(0., N.y)) * uv1.y;
#endif

//Ray Sun
#ifdef RAY_SUN
#ifndef BLEND
diffuse.rgb += mix(vec3(0.), Ray_sun(wpos).rgb, max(0.0, N.y)) * day * (1. - rain2) * uv1.y;
#endif
#endif

//Water
if((color.r * 1.1 < color.b) && (color.b > color.g * 1.05) && (color.g > color.r) && (color.a > 0.0))
{
#ifdef REALISTIC_WATER
   diffuse = water_color(position, wpos, night, sun);
#endif
   float smooth_pos = length(-wpos)/RENDER_DISTANCE;
   diffuse.a += inColor.a * (2. * smooth_pos);
}

//Caustic
#ifdef CAUSTIC
if(Unwater == true && Nwater == false && uv1.y < .875)
{
   diffuse.rgb = caustic(position, diffuse.rgb);
}
#endif

//DAY FOG
float fog_pos = clamp(length(-wpos)/FAR_CHUNKS_DISTANCE * .4, 0., .5);
vec4 fog = FOG_COLOR;
fog.rgb *= vec3(1., 1.15, 1.3);
diffuse = mix(diffuse, mix(diffuse, fog, fog_pos), day);

#ifdef FOG
if(Unwater == true)
{
	diffuse.rgb = mix(diffuse.rgb, diffuse.rgb, fogColor.a);
}else{
diffuse.rgb = mix( diffuse.rgb, diffuse.rgb, diffuse.a);
}

if(Nether == true)diffuse.rgb = mix(diffuse.rgb, fc.rgb, fogColor.a);
#endif

	gl_FragColor = diffuse;
	
#endif // BYPASS_PIXEL_SHADER
}
