// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

/*
Unreal Graphic Shader 1.0

Dev : Dimas Adiyaksa

Facebook : Dimas Adiyaksa
Fanspage Facebook : Unreal Graphic Shader PE
Instagram : @dimas.adiyaksa
Twitter : @dimasadiyaksa2
Discord : Dimas 956
*/

/*
=========== DONT STEAL ANYTHING !!! ==========
*/

#include "vertexVersionCentroid.h"
#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		_centroid out vec2 uv0;
		_centroid out vec2 uv1;
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

#ifndef BYPASS_PIXEL_SHADER
	varying vec4 color;
#endif

#ifdef FOG
	varying vec4 fogColor;
#endif

varying highp vec3 position;
varying highp vec3 wpos;

#include "uniformWorldConstants.h"
#include "uniformPerFrameConstants.h"
#include "uniformShaderConstants.h"
#include "uniformRenderChunkConstants.h"

#include "lib/Water.glsl"

#include "Settings/Setting.ugspe"

attribute POS4 POSITION;
attribute vec4 COLOR;
attribute vec2 TEXCOORD_0;
attribute vec2 TEXCOORD_1;

const float rA = 1.0;
const float rB = 1.0;
const vec3 UNIT_Y = vec3(0,1,0);
const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange

void main()
{
    POS4 worldPos;
#ifdef AS_ENTITY_RENDERER
		POS4 pos = WORLDVIEWPROJ * POSITION;
		worldPos = pos;
#else
    worldPos.xyz = (POSITION.xyz * CHUNK_ORIGIN_AND_SCALE.w) + CHUNK_ORIGIN_AND_SCALE.xyz;
    worldPos.w = 1.0;

    // Transform to view space before projection instead of all at once to avoid floating point errors
    // Not required for entities because they are already offset by camera translation before rendering
    // World position here is calculated above and can get huge
    POS4 pos = WORLDVIEW * worldPos;
    pos = PROJ * pos;
#endif
    gl_Position = pos;
    wpos = worldPos.xyz;
    position = POSITION.xyz;
 
#ifndef BYPASS_PIXEL_SHADER
    uv0 = TEXCOORD_0;
    uv1 = TEXCOORD_1;
	color = COLOR;
#endif

//Waving Leave & Grass
vec3 p;
#ifdef WAVING_LEAVES
#ifdef ALPHA_TEST
p = position;
if(color.g * 2. > color.b + color.r && color.a > 0.){
gl_Position.x += sin(TIME * 4. + p.x * 1.7 + p.z * 1.7 + p.y  + sin(TIME * 2.5 + p.x + p.z + p.y * .7)) * cos(TIME + p.z + p.z + p.y * .6) * cos(fract(p.x)) * .05 * uv1.y;
}
if(color.g * 2. > color.b + color.r && color.a == 0.)
{
gl_Position.x += sin(TIME * 3. + p.x + p.z * cos(TIME * .5 + p.z + p.y)) * .01 * uv1.y;
}
#else
p = wpos;
#endif
#endif
#ifdef WAVING
if(FOG_CONTROL.x == 0.)
{
float rad = clamp(length(-wpos)/RENDER_DISTANCE * 5., 0., 1.);
float wav_pos = mix(0., sin(TIME * 3.5 + p.x + p.z + p.x * 3.) * .15, rad);
if(uv1.y < .875){
gl_Position.x += wav_pos;
}else{
gl_Position.y += wav_pos * 1.7;
}
}
#endif




///// find distance from the camera

#if defined(FOG) || defined(BLEND)
	#ifdef FANCY
		vec3 relPos = -worldPos.xyz;
		float cameraDepth = length(relPos);
	#else
		float cameraDepth = pos.z;
	#endif
#endif

///// apply fog

#ifdef FOG
  float len = cameraDepth / RENDER_DISTANCE;
	#ifdef ALLOW_FADE
		len += RENDER_CHUNK_FOG_ALPHA;
	#endif


  fogColor.rgb = FOG_COLOR.rgb;
	fogColor.a = clamp((len - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 0.0, 1.0);
#endif

///// blended layer (mostly water) magic
#ifdef BLEND
	//Mega hack: only things that become opaque are allowed to have vertex-driven transparency in the Blended layer...
	//to fix this we'd need to find more space for a flag in the vertex format. color.a is the only unused part
	bool shouldBecomeOpaqueInTheDistance = color.a < 0.95;
	if(shouldBecomeOpaqueInTheDistance) {
		#ifdef FANCY  /////enhance water
			float cameraDist = cameraDepth / FAR_CHUNKS_DISTANCE;
			color = COLOR;
		#else
			// Completely insane, but if I don't have these two lines in here, the water doesn't render on a Nexus 6
			vec4 surfColor = vec4(color.rgb, 1.0);
			color = surfColor;
				
			vec3 relPos = -worldPos.xyz;
			float camDist = length(relPos);
			float cameraDist = camDist / FAR_CHUNKS_DISTANCE;
		#endif //FANCY
		
		float alphaFadeOut = clamp(cameraDist, 0.0, 1.0);
		color.a = mix(color.a, 1.0, alphaFadeOut);
	}
#endif

#ifndef BYPASS_PIXEL_SHADER
	#ifndef FOG
		// If the FOG_COLOR isn't used, the reflection on NVN fails to compute the correct size of the constant buffer as the uniform will also be gone from the reflection data
		color.rgb += FOG_COLOR.rgb * 0.000001;
	#endif
#endif
}
